import java.lang.Math;
import java.io.*;

class pi_fftcs
{

	/*
	---- calculation of PI(= 3.14159...) using FFT and AGM ----
		by T.Ooura, Nov. 1999.
		converted to Java by D. Hazeghi, Aug. 2004
	
	Example compilation:
		SUN: javac pi_fftcs.java

	Invokation:
		SUN: java pi_fftcs
	*/
	
	/* version number */
	static final String PI_FFTC_VER = "ver. LG1.1.2-MP1.5.2a.memsave";

	/* set to true for (mostly) quiet output */
	static final boolean QUIET_OUT = false;

	/* Please check the following constants before compiling */
	static final double DBL_ERROR_MARGIN = 0.4;	/* must be < 0.5 */

	/* maximum number of digits per unit: can be as large as INT_MAX */
	static final int DGTINT_MAX = 32767;

	/* constants that determine how pi is printed out */
	
	/* digits per group */
	static final int DGT_PACK = 10;
	
	/* groups per line */
	static final int DGT_PACK_LINE = 5;
	
	/* lines per block */
	static final int DGT_LINE_BLOCK = 20;

	/* copied from math.h */
	static final double DBL_EPSILON = 2.2204460492503131e-16;

	/* main */
	public static void main(String args[])
	{
		int nfft, log2_nfft, radix, log10_radix, n,
		npow, nprc;
		double err;
		int a[], b[], c[], e[], i1[], i2[];
		double d1[], d2[], d3[];
		String filename; // for the output
		long start_time; // clock at starting time, in milliseconds
		double elap_time, loop_time; // record the elapsed times
		BufferedWriter f_out; // file used to output pi
		char dgt[]; // store the computed digits

		try {
			BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
			
		if (!QUIET_OUT)
			System.out.println("Calculation of PI using FFT and AGM, " + PI_FFTC_VER);
			
		// wrong number of arguments
		if (args.length != 1)
		{
			System.out.println("Usage: java pi_fftcs digits");
			nfft = java.lang.Integer.parseInt(input.readLine(),10);
		}
		else
			nfft = java.lang.Integer.parseInt(args[0], 10);

		if (!QUIET_OUT)
			System.out.println("initializing...");

		nfft /= 4;
		
		// start the timer
		start_time = System.currentTimeMillis();
		
		// compute fft length
		for (log2_nfft = 1; (1 << log2_nfft) < nfft; log2_nfft++);
		nfft = 1 << log2_nfft;
		
		// allocate space
		n = nfft + 2;
		a = new int[2 + n];
		b = new int[2 + n];
		c = new int[2 + n];
		e = new int[2 + n];
		i1 = new int[2 + n];
		i2 = new int[2 + n];
		d1 = new double[2 + nfft];
		d2 = new double[2 + nfft];
		d3 = new double[2 + nfft];
		dgt = new char[4 * (2 + nfft)];

		/* ---- radix test ---- */
		log10_radix = 1;
		radix = 10;
		err = mp_mul_radix_test(n, radix, nfft, d1);
		err += DBL_EPSILON * (n * radix * radix / 4);
		while (100 * err < DBL_ERROR_MARGIN && radix <= DGTINT_MAX / 20)
		{
			err *= 100;
			log10_radix++;
			radix *= 10;
		}
		
		// program data
		if (!QUIET_OUT)
		{
			System.out.println("nfft= " + nfft);
			System.out.println("radix= " + radix);
			System.out.println("error_margin= " + err);
			System.out.println("calculating " + log10_radix * (n - 2) + " digits of PI...");
		}
		/*
		 * ---- a formula based on the AGM (Arithmetic-Geometric Mean) ----
		 *   c = sqrt(0.125);
		 *   a = 1 + 3 * c;
		 *   b = sqrt(a);
		 *   e = b - 0.625;
		 *   b = 2 * b;
		 *   c = e - c;
		 *   a = a + e;
		 *   npow = 4;
		 *   do {
		 *       npow = 2 * npow;
		 *       e = (a + b) / 2;
		 *       b = sqrt(a * b);
		 *       e = e - b;
		 *       b = 2 * b;
		 *       c = c - e;
		 *       a = e + b;
		 *   } while (e > SQRT_SQRT_EPSILON);
		 *   e = e * e / 4;
		 *   a = a + b;
		 *   pi = (a * a - e - e / 2) / (a * c - e) / npow;
		 * ---- modification ----
		 *   This is a modified version of Gauss-Legendre formula
		 *   (by T.Ooura). It is faster than original version.
		 * ---- reference ----
		 *   1. E.Salamin,
		 *      Computation of PI Using Arithmetic-Geometric Mean,
		 *      Mathematics of Computation, Vol.30 1976.
		 *   2. R.P.Brent,
		 *      Fast Multiple-Precision Evaluation of Elementary Functions,
		 *      J. ACM 23 1976.
		 *   3. D.Takahasi, Y.Kanada,
		 *      Calculation of PI to 51.5 Billion Decimal Digits on
		 *      Distributed Memoriy Parallel Processors,
		 *      Transactions of Information Processing Society of Japan,
		 *      Vol.39 No.7 1998.
		 *   4. T.Ooura,
		 *      Improvement of the PI Calculation Algorithm and
		 *      Implementation of Fast Multiple-Precision Computation,
		 *      Information Processing Society of Japan SIG Notes,
		 *      98-HPC-74, 1998.
		 */
		 
		/* ---- c = 1 / sqrt(8) ---- */
		mp_invisqrt(n, radix, 8, c, i1, i2, nfft, d1, d2);
		
		/* ---- a = 1 + 3 * c ---- */
		mp_imul(n, radix, c, 3, e);
		mp_sscanf(n, log10_radix, "1", a);
		mp_add(n, radix, a, e, a);
		
		/* ---- b = sqrt(a) ---- */
		mp_sqrt(n, radix, a, b, i1, i2, nfft, d1, d2);
		
		/* ---- e = b - 0.625 ---- */
		mp_sscanf(n, log10_radix, "0.625", e);
		mp_sub(n, radix, b, e, e);
		
		/* ---- b = 2 * b ---- */
		mp_add(n, radix, b, b, b);
		
		/* ---- c = e - c ---- */
		mp_sub(n, radix, e, c, c);
		
		/* ---- a = a + e ---- */
		mp_add(n, radix, a, e, a);
		
		if (!QUIET_OUT)
			System.out.println("AGM iteration");

		npow = 4;
		elap_time = ((float) (System.currentTimeMillis() - start_time)) / 1000;

		do {
			long start_loop_time = System.currentTimeMillis();
			npow *= 2;
			
			/* ---- e = (a + b) / 2 ---- */
			mp_add(n, radix, a, b, e);
			mp_idiv_2(n, radix, e, e);
			
			/* ---- b = sqrt(a * b) ---- */
			mp_mul(n, radix, a, b, a, i1, nfft, d1, d2, d3);
			mp_sqrt(n, radix, a, b, i1, i2, nfft, d1, d2);
			
			/* ---- e = e - b ---- */
			mp_sub(n, radix, e, b, e);
			
			/* ---- b = 2 * b ---- */
			mp_add(n, radix, b, b, b);
			
			/* ---- c = c - e ---- */
			mp_sub(n, radix, c, e, c);
			
			/* ---- a = e + b ---- */
			mp_add(n, radix, e, b, a);
			
			/* ---- convergence check ---- */
			nprc = -e[1];
			if (e[0] == 0) nprc = n;

			loop_time = ((float) (System.currentTimeMillis() - start_loop_time)) / 1000;
			elap_time += loop_time;
			if (!QUIET_OUT)
				System.out.println("precision= " + 4 * nprc * log10_radix + ": " + doubleToString(loop_time) + " sec");
		} while (4 * nprc <= n);
		
		start_time = System.currentTimeMillis();
		
		/* ---- e = e * e / 4 (half precision) ---- */
		mp_idiv_2(n, radix, e, e);
		mp_squh(n, radix, e, e, nfft, d1);
		
		/* ---- a = a + b ---- */
		mp_add(n, radix, a, b, a);
		
		/* ---- a = (a * a - e - e / 2) / (a * c - e) / npow ---- */
		mp_mulhf(n, radix, a, c, c, i1, nfft, d1, d2);
		mp_sub(n, radix, c, e, c);
		mp_inv(n, radix, c, b, i1, i2, nfft, d2, d3);
		mp_squhf_use_infft(n, radix, d1, a, a, i1, nfft, d2);
		mp_sub(n, radix, a, e, a);
		mp_idiv_2(n, radix, e, e);
		mp_sub(n, radix, a, e, a);
		mp_mul(n, radix, a, b, a, i1, nfft, d1, d2, d3);
		mp_idiv(n, radix, a, npow, a);
		
		/* ---- output ---- */
		mp_sprintf(n - 1, log10_radix, a, dgt);
		elap_time += ((float) (System.currentTimeMillis() - start_time)) / 1000;

		filename = "pi" + (log10_radix * (n - 2)) + ".txt";
		try {
			f_out = new BufferedWriter(new FileWriter(filename));
			if (!QUIET_OUT)
				System.out.println("writing " + filename);
			mp_outfile(f_out, dgt);
			f_out.close();
		} catch(IOException ioe) { }

		/* ---- difftime ---- */
		System.out.println(doubleToString(elap_time) + " sec. (real time)");
		
		if(args.length != 1)
		{
			System.out.println("Hit RETURN to exit.");
			input.readLine();
		}

		} catch (IOException f) { }
	}

	// convert double to string with only 3 digits after decimal point
	static String doubleToString(double d) {
		String s = Double.toString(d);
		int pos, end = s.length();
		if(end >= s.indexOf('.') + 3) pos = s.indexOf('.')+3;
		else pos = end;
		return s.substring(0, pos);
	}

	static void mp_outfile(BufferedWriter f_out, char dgt[]) {
		int j = 0, k = 0, l = 0;
		int count = 0;
		try {
			do {
				if (!Character.isDigit(dgt[count])) {
					if (Character.isLetter(dgt[count])) {
						f_out.newLine();
						f_out.newLine();
					}
					f_out.write(dgt, count, 1);
					f_out.newLine();
					f_out.newLine();
					j = 0;
					k = 0;
					l = 0;
					continue;
				}
				f_out.write(dgt, count, 1);
				if (++j >= DGT_PACK) {
					j = 0;
					if (++k >= DGT_PACK_LINE) {
						k = 0;
						f_out.newLine();
						if (++l >= DGT_LINE_BLOCK) {
							l = 0;
							f_out.newLine();
						}
					} else {
						f_out.write(' ');
					}
				}
			}
			while (dgt[count++] != '\0' && dgt[count] != 'e');
			f_out.newLine();
			f_out.write(dgt, count, 2);
			f_out.newLine();
		} catch(IOException ioe) {
		}
	}


	/* -------- multiple precision routines -------- */


	/* -------- mp_load routines -------- */


	static void mp_load_0(int n, int radix, int out[]) {
		int j;
		out[0] = 0;
		out[1] = 0;
		for (j = 2; j <= n + 1; j++)
			out[j] = 0;
	}


	static void mp_load_1(int n, int radix, int out[]) {
		int j;
		out[0] = 1;
		out[1] = 0;
		out[2] = 1;
		for (j = 3; j <= n + 1; j++)
			out[j] = 0;
	}


	static void mp_round(int n, int radix, int m, int inout[]) {
		int j, x;

		if (m < n) {
			for (j = n + 1; j > m + 2; j--) {
				inout[j] = 0;
			}
			x = 2 * inout[m + 2];
			inout[m + 2] = 0;
			if (x >= radix) {
				for (j = m + 1; j >= 2; j--) {
					x = inout[j] + 1;
					if (x < radix) {
						inout[j] = (int) x;
						break;
					}
					inout[j] = 0;
				}
				if (x >= radix) {
					inout[2] = 1;
					inout[1]++;
				}
			}
		}
	}


	/* -------- mp_add routines -------- */


	static int mp_cmp(int n, int radix, int in1[], int in2[]) {

		if (in1[0] > in2[0]) {
			return 1;
		} else if (in1[0] < in2[0]) {
			return -1;
		}
		return in1[0] * mp_unsgn_cmp(n, 1, in1, in2);
	}


	static void mp_add(int n, int radix, int in1[], int in2[], int out[]) {
		int outsgn, outexp, expdif;

		expdif = in1[1] - in2[1];
		outexp = in1[1];
		if (expdif < 0) {
			outexp = in2[1];
		}
		outsgn = in1[0] * in2[0];
		if (outsgn >= 0) {
			if (outsgn > 0) {
				outsgn = in1[0];
			} else {
				outsgn = in1[0] + in2[0];
				outexp = in1[1] + in2[1];
				expdif = 0;
			}
			if (expdif >= 0) {
				outexp += mp_unexp_add(n, 2, radix, expdif, in1, in2, out);
			} else {
				outexp += mp_unexp_add(n, 2, radix, -expdif, in2, in1, out);
			}
		} else {
			outsgn = mp_unsgn_cmp(n, 1, in1, in2);
			if (outsgn >= 0) {
				expdif = mp_unexp_sub(n, 2, radix, expdif, in1, in2, out);
			} else {
				expdif = mp_unexp_sub(n, 2, radix, -expdif, in2, in1, out);
			}
			outexp -= expdif;
			outsgn *= in1[0];
			if (expdif == n) {
				outsgn = 0;
			}
		}
		if (outsgn == 0) {
			outexp = 0;
		}
		out[0] = outsgn;
		out[1] = outexp;
	}


	static void mp_sub(int n, int radix, int in1[], int in2[], int out[]) {
		int outsgn, outexp, expdif;

		expdif = in1[1] - in2[1];
		outexp = in1[1];
		if (expdif < 0) {
			outexp = in2[1];
		}
		outsgn = in1[0] * in2[0];
		if (outsgn <= 0) {
			if (outsgn < 0) {
				outsgn = in1[0];
			} else {
				outsgn = in1[0] - in2[0];
				outexp = in1[1] + in2[1];
				expdif = 0;
			}
			if (expdif >= 0) {
				outexp += mp_unexp_add(n, 2, radix, expdif, in1, in2, out);
			} else {
				outexp += mp_unexp_add(n, 2, radix, -expdif, in2, in1, out);
			}
		} else {
			outsgn = mp_unsgn_cmp(n, 1, in1, in2);
			if (outsgn >= 0) {
				expdif = mp_unexp_sub(n, 2, radix, expdif, in1, in2, out);
			} else {
				expdif = mp_unexp_sub(n, 2, radix, -expdif, in2, in1, out);
			}
			outexp -= expdif;
			outsgn *= in1[0];
			if (expdif == n) {
				outsgn = 0;
			}
		}
		if (outsgn == 0) {
			outexp = 0;
		}
		out[0] = outsgn;
		out[1] = outexp;
	}


	/* -------- mp_add child routines -------- */


	static int mp_unsgn_cmp(int n, int base, int in1[], int in2[]) {
		int j, cmp;
		cmp = in1[base] - in2[base];
		for (j = 1; j <= n && cmp == 0; j++) {
			cmp = in1[base + j] - in2[base + j];
		}
		if (cmp > 0) {
			cmp = 1;
		} else if (cmp < 0) {
			cmp = -1;
		}
		return cmp;
	}


	static int mp_unexp_add(int n, int base, int radix, int expdif,
		int in1[], int in2[], int out[]) {
		int j, x, carry;

		carry = 0;
		if (expdif == 0 && in1[base + 0] + in2[base + 0] >= radix) {
			x = in1[base + n - 1] + in2[base + n - 1];
			carry = x >= radix ? -1 : 0;
			for (j = n - 1; j > 0; j--) {
				x = in1[base + j - 1] + in2[base + j - 1] - carry;
				carry = x >= radix ? -1 : 0;
				out[base + j] = (int) (x - (radix & carry));
			}
			out[base] = (int) -carry;
		} else {
			if (expdif > n) {
				expdif = n;
			}
			for (j = n - 1; j >= expdif; j--) {
				x = in1[base + j] + in2[base + j - expdif] - carry;
				carry = x >= radix ? -1 : 0;
				out[base + j] = (int) (x - (radix & carry));
			}
			for (j = expdif - 1; j >= 0; j--) {
				x = in1[base + j] - carry;
				carry = x >= radix ? -1 : 0;
				out[base + j] = (int) (x - (radix & carry));
			}
			if (carry != 0) {
				for (j = n - 1; j > 0; j--) {
					out[base + j] = out[base + j - 1];
				}
				out[base] = (int) -carry;
			}
		}
		return -carry;
	}


	static int mp_unexp_sub(int n, int base, int radix, int expdif,
		int in1[], int in2[], int out[]) {
		int j, x, borrow, ncancel;

		if (expdif > n) {
			expdif = n;
		}
		borrow = 0;
		for (j = n - 1; j >= expdif; j--) {
			x = in1[base + j] - in2[base + j - expdif] + borrow;
			borrow = x < 0 ? -1 : 0;
			out[base + j] = (int) (x + (radix & borrow));
		}
		for (j = expdif - 1; j >= 0; j--) {
			x = in1[base + j] + borrow;
			borrow = x < 0 ? -1 : 0;
			out[base + j] = (int) (x + (radix & borrow));
		}
		ncancel = 0;
		for (j = 0; j < n && out[base + j] == 0; j++) {
			ncancel = j + 1;
		}
		if (ncancel > 0 && ncancel < n) {
			for (j = 0; j < n - ncancel; j++) {
				out[base + j] = out[base + j + ncancel];
			}
			for (j = n - ncancel; j < n; j++) {
				out[base + j] = 0;
			}
		}
		return ncancel;
	}


	/* -------- mp_imul routines -------- */


	static void mp_imul(int n, int radix, int in1[], int in2, int out[]) {

		if (in2 > 0) {
			out[0] = in1[0];
		} else if (in2 < 0) {
			out[0] = -in1[0];
			in2 = -in2;
		} else {
			out[0] = 0;
		}
		mp_unsgn_imul(n, 1, radix, in1, in2, out);
		if (out[0] == 0) {
			out[1] = 0;
		}
	}


	static int mp_idiv(int n, int radix, int in1[], int in2, int out[]) {

		if (in2 == 0) {
			return -1;
		}
		if (in2 > 0) {
			out[0] = in1[0];
		} else {
			out[0] = -in1[0];
			in2 = -in2;
		}
		if (in1[0] == 0) {
			mp_load_0(n, radix, out);
			return 0;
		}
		mp_unsgn_idiv(n, 1, radix, in1, in2, out);
		return 0;
	}


	static void mp_idiv_2(int n, int radix, int in[], int out[]) {
		int j, ix, carry, shift;

		out[0] = in[0];
		shift = 0;
		if (in[2] == 1) {
			shift = 1;
		}
		out[1] = in[1] - shift;
		carry = -shift;
		for (j = 2; j <= n + 1 - shift; j++) {
			ix = in[j + shift] + (radix & carry);
			carry = -(ix & 1);
			out[j] = (int) (ix >> 1);
		}
		if (shift > 0) {
			out[n + 1] = (int) ((radix & carry) >> 1);
		}
	}


	/* -------- mp_imul child routines -------- */


	static void mp_unsgn_imul(int n, int base, double dradix, int in1[], double din2, int out[]) {
		int j, carry, shift;
		double x, d1_radix;

		d1_radix = 1.0 / dradix;
		carry = 0;
		for (j = n; j >= 1; j--) {
			x = din2 * in1[base + j] + carry + 0.5;
			carry = (int) (d1_radix * x);
			out[base + j] = (int) (x - dradix * carry);
		}
		shift = 0;
		x = carry + 0.5;
		while (x > 1) {
			x *= d1_radix;
			shift++;
		}
		out[base] = in1[base] + shift;
		if (shift > 0) {
			while (shift > n) {
				carry = (int) (d1_radix * carry + 0.5);
				shift--;
			}
			for (j = n; j >= shift + 1; j--) {
				out[base + j] = out[base + j - shift];
			}
			for (j = shift; j >= 1; j--) {
				x = carry + 0.5;
				carry = (int) (d1_radix * x);
				out[base + j] = (int) (x - dradix * carry);
			}
		}
	}


	static void mp_unsgn_idiv(int n, int base, double dradix, int in1[], double din2, int out[]) {
		int j, ix, carry, shift;
		double x, d1_in2;

		d1_in2 = 1.0 / din2;
		shift = 0;
		x = 0;
		do {
			shift++;
			x *= dradix;
			if (shift <= n) {
				x += in1[base + shift];
			}
		}
		while (x < din2 - 0.5);
		x += 0.5;
		ix = (int) (d1_in2 * x);
		carry = (int) (x - din2 * ix);
		out[base + 1] = (int) ix;
		shift--;
		out[base] = in1[base] - shift;
		if (shift >= n) {
			shift = n - 1;
		}
		for (j = 2; j <= n - shift; j++) {
			x = in1[base + j + shift] + dradix * carry + 0.5;
			ix = (int) (d1_in2 * x);
			carry = (int) (x - din2 * ix);
			out[base + j] = (int) ix;
		}
		for (j = n - shift + 1; j <= n; j++) {
			x = dradix * carry + 0.5;
			ix = (int) (d1_in2 * x);
			carry = (int) (x - din2 * ix);
			out[base + j] = (int) ix;
		}
	}


	/* -------- mp_mul routines -------- */


	static double mp_mul_radix_test(int n, int radix, int nfft, double tmpfft[]) {
		int j, ndata, radix_2;

		ndata = (nfft >> 1) + 1;
		if (ndata > n) {
			ndata = n;
		}
		tmpfft[nfft + 1] = radix - 1;
		for (j = nfft; j > ndata; j--) {
			tmpfft[j] = 0;
		}
		radix_2 = (radix + 1) / 2;
		for (j = ndata; j > 2; j--) {
			tmpfft[j] = radix_2;
		}
		tmpfft[2] = radix;
		tmpfft[1] = radix - 1;
		tmpfft[0] = 0;
		mp_mul_csqu(nfft, tmpfft);
		return 2 * mp_mul_d2i_test(radix, nfft, tmpfft);
	}


	static void mp_mul(int n, int radix, int in1[], int in2[], int out[],
		int tmp[], int nfft, double tmp1fft[], double tmp2fft[],
		double tmp3fft[]) {

		int n_h, shift;

		shift = (nfft >> 1) + 1;
		while (n > shift) {
			if (in1[shift + 2] + in2[shift + 2] != 0) {
				break;
			}
			shift++;
		}
		n_h = n / 2 + 1;
		if (n_h < n - shift) {
			n_h = n - shift;
		}
		/* ---- tmp3fft = (upper) in1 * (lower) in2 ---- */
		mp_mul_i2d(n, radix, nfft, 0, in1, tmp1fft);
		mp_mul_i2d(n, radix, nfft, shift, in2, tmp3fft);
		mp_mul_cmul_nt_out(nfft, tmp1fft, tmp3fft);
		/* ---- tmp = (upper) in1 * (upper) in2 ---- */
		mp_mul_i2d(n, radix, nfft, 0, in2, tmp2fft);
		mp_mul_cmul_nt_d2(nfft, tmp2fft, tmp1fft);
		mp_mul_d2i(n, radix, nfft, tmp1fft, tmp);
		/* ---- tmp3fft += (upper) in2 * (lower) in1 ---- */
		mp_mul_i2d(n, radix, nfft, shift, in1, tmp1fft);
		mp_mul_cmul_nt_d1_add(nfft, tmp2fft, tmp1fft, tmp3fft);
		/* ---- out = tmp + tmp3fft ---- */
		mp_mul_d2i(n_h, radix, nfft, tmp3fft, out);
		mp_add(n, radix, out, tmp, out);
	}


	static void mp_squ(int n, int radix, int in[], int out[], int tmp[],
		int nfft, double tmp1fft[], double tmp2fft[]) {
		int n_h, shift;

		shift = (nfft >> 1) + 1;
		while (n > shift) {
			if (in[shift + 2] != 0) {
				break;
			}
			shift++;
		}
		n_h = n / 2 + 1;
		if (n_h < n - shift) {
			n_h = n - shift;
		}
		/* ---- tmp = 2 * (upper) in * (lower) in ---- */
		mp_mul_i2d(n, radix, nfft, 0, in, tmp1fft);
		mp_mul_i2d(n, radix, nfft, shift, in, tmp2fft);
		mp_mul_cmul(nfft, tmp1fft, tmp2fft);
		mp_mul_d2i(n_h, radix, nfft, tmp2fft, tmp);
		mp_add(n_h, radix, tmp, tmp, tmp);
		/* ---- out = tmp + ((upper) in)^2 ---- */
		mp_mul_csqu_nt_d1(nfft, tmp1fft);
		mp_mul_d2i(n, radix, nfft, tmp1fft, out);
		mp_add(n, radix, out, tmp, out);
	}


	static void mp_mulhf(int n, int radix, int in1[], int in2[], int out[],
		int tmp[], int nfft, double in1fft[], double tmpfft[]) {
		int n_h, shift;

		shift = (nfft >> 1) + 1;
		while (n > shift) {
			if (in2[shift + 2] != 0) {
				break;
			}
			shift++;
		}
		n_h = n / 2 + 1;
		if (n_h < n - shift) {
			n_h = n - shift;
		}
		/* ---- tmp = (upper) in1 * (upper) in2 ---- */
		mp_mul_i2d(n, radix, nfft, 0, in1, in1fft);
		mp_mul_i2d(n, radix, nfft, 0, in2, tmpfft);
		mp_mul_cmul(nfft, in1fft, tmpfft);
		mp_mul_d2i(n, radix, nfft, tmpfft, tmp);
		/* ---- out = tmp + (upper) in1 * (lower) in2 ---- */
		mp_mul_i2d(n, radix, nfft, shift, in2, tmpfft);
		mp_mul_cmul_nt_d1(nfft, in1fft, tmpfft);
		mp_mul_d2i(n_h, radix, nfft, tmpfft, out);
		mp_add(n, radix, out, tmp, out);
	}


	static void mp_mulhf_use_in1fft(int n, int radix, double in1fft[], int in2[],
		int out[], int tmp[], int nfft, double tmpfft[]) {
		int n_h, shift;

		shift = (nfft >> 1) + 1;
		while (n > shift) {
			if (in2[shift + 2] != 0) {
				break;
			}
			shift++;
		}
		n_h = n / 2 + 1;
		if (n_h < n - shift) {
			n_h = n - shift;
		}
		/* ---- tmp = (upper) in1fft * (upper) in2 ---- */
		mp_mul_i2d(n, radix, nfft, 0, in2, tmpfft);
		mp_mul_cmul_nt_d1(nfft, in1fft, tmpfft);
		mp_mul_d2i(n, radix, nfft, tmpfft, tmp);
		/* ---- out = tmp + (upper) in1 * (lower) in2 ---- */
		mp_mul_i2d(n, radix, nfft, shift, in2, tmpfft);
		mp_mul_cmul_nt_d1(nfft, in1fft, tmpfft);
		mp_mul_d2i(n_h, radix, nfft, tmpfft, out);
		mp_add(n, radix, out, tmp, out);
	}


	static void mp_squhf_use_infft(int n, int radix, double infft[], int in[],
		int out[], int tmp[], int nfft, double tmpfft[]) {
		int n_h, shift;

		shift = (nfft >> 1) + 1;
		while (n > shift) {
			if (in[shift + 2] != 0) {
				break;
			}
			shift++;
		}
		n_h = n / 2 + 1;
		if (n_h < n - shift) {
			n_h = n - shift;
		}
		/* ---- tmp = (upper) infft * (lower) in ---- */
		mp_mul_i2d(n, radix, nfft, shift, in, tmpfft);
		mp_mul_cmul_nt_d1(nfft, infft, tmpfft);
		mp_mul_d2i(n_h, radix, nfft, tmpfft, tmp);
		/* ---- out = tmp + ((upper) infft)^2 ---- */
		mp_mul_csqu_nt_d1(nfft, infft);
		mp_mul_d2i(n, radix, nfft, infft, out);
		mp_add(n, radix, out, tmp, out);
	}


	static void mp_mulh(int n, int radix, int in1[], int in2[], int out[],
		int nfft, double in1fft[], double outfft[]) {

		mp_mul_i2d(n, radix, nfft, 0, in1, in1fft);
		mp_mul_i2d(n, radix, nfft, 0, in2, outfft);
		mp_mul_cmul(nfft, in1fft, outfft);
		mp_mul_d2i(n, radix, nfft, outfft, out);
	}


	static void mp_mulh_use_in1fft(int n, int radix, double in1fft[],
		int shift, int in2[], int out[], int nfft,
		double outfft[]) {
		while (n > shift) {
			if (in2[shift + 2] != 0) {
				break;
			}
			shift++;
		}
		mp_mul_i2d(n, radix, nfft, shift, in2, outfft);
		mp_mul_cmul_nt_d1(nfft, in1fft, outfft);
		mp_mul_d2i(n, radix, nfft, outfft, out);
	}


	static void mp_squh(int n, int radix, int in[], int out[], int nfft, double outfft[]) {

		mp_mul_i2d(n, radix, nfft, 0, in, outfft);
		mp_mul_csqu(nfft, outfft);
		mp_mul_d2i(n, radix, nfft, outfft, out);
	}


	static void mp_squh_save_infft(int n, int radix, int in[], int out[],
		int nfft, double infft[], double outfft[]) {

		mp_mul_i2d(n, radix, nfft, 0, in, infft);
		mp_mul_csqu_save_d1(nfft, infft, outfft);
		mp_mul_d2i(n, radix, nfft, outfft, out);
	}


	static void mp_squh_use_in1fft(int n, int radix, double inoutfft[], int out[], int nfft) {

		mp_mul_csqu_nt_d1(nfft, inoutfft);
		mp_mul_d2i(n, radix, nfft, inoutfft, out);
	}


	/* -------- mp_mul child routines -------- */


	static void mp_mul_i2d(int n, int radix, int nfft, int shift, int in[], double dout[]) {
		int j, x, carry, ndata, radix_2, topdgt;

		ndata = 0;
		topdgt = 0;
		if (n > shift) {
			topdgt = in[shift + 2];
			ndata = (nfft >> 1) + 1;
			if (ndata > n - shift) {
				ndata = n - shift;
			}
		}
		dout[nfft + 1] = in[0] * topdgt;
		for (j = nfft; j > ndata; j--) {
			dout[j] = 0;
		}
		/* ---- abs(dout[j]) <= radix/2 (to keep FFT precision) ---- */
		if (ndata > 1) {
			radix_2 = radix / 2;
			carry = 0;
			for (j = ndata + 1; j > 3; j--) {
				x = in[j + shift] - carry;
				carry = x >= radix_2 ? -1 : 0;
				dout[j - 1] = x - (radix & carry);
			}
			dout[2] = in[shift + 3] - carry;
		}
		dout[1] = topdgt;
		dout[0] = in[1] - shift;
	}


	static void mp_mul_cmul(int nfft, double d1[], double d2[]) {
		double xr, xi;

		fftsg_h.cdft(nfft, 1, 1, d1);
		fftsg_h.cdft(nfft, 1, 1, d2);
		d2[0] += d1[0];
		xr = d1[1] * d2[1] + d1[2] * d2[2];
		xi = d1[1] * d2[2] + d1[2] * d2[1];
		d2[1] = xr;
		d2[2] = xi;
		if (nfft > 2) {
			mp_mul_rcmul(nfft, 1, d1, d2);
		}
		d2[nfft + 1] *= d1[nfft + 1];
		fftsg_h.cdft(nfft, 1, -1, d2);
	}


	static void mp_mul_cmul_nt_d1(int nfft, double d1[], double d2[]) {
		double xr, xi;

		fftsg_h.cdft(nfft, 1, 1, d2);
		d2[0] += d1[0];
		xr = d1[1] * d2[1] + d1[2] * d2[2];
		xi = d1[1] * d2[2] + d1[2] * d2[1];
		d2[1] = xr;
		d2[2] = xi;
		if (nfft > 2) {
			mp_mul_rcmul_nt_in1(nfft, 1, d1, d2);
		}
		d2[nfft + 1] *= d1[nfft + 1];
		fftsg_h.cdft(nfft, 1, -1, d2);
	}


	static void mp_mul_cmul_nt_d2(int nfft, double d1[], double d2[]) {
		double xr, xi;

		fftsg_h.cdft(nfft, 1, 1, d1);
		d2[0] += d1[0];
		xr = d1[1] * d2[1] + d1[2] * d2[2];
		xi = d1[1] * d2[2] + d1[2] * d2[1];
		d2[1] = xr;
		d2[2] = xi;
		if (nfft > 2) {
			mp_mul_rcmul_nt_in2(nfft, 1, d1, d2);
		}
		d2[nfft + 1] *= d1[nfft + 1];
		fftsg_h.cdft(nfft, 1, -1, d2);
	}


	static void mp_mul_cmul_nt_out(int nfft, double d1[], double d2[]) {
		double xr, xi;

		fftsg_h.cdft(nfft, 1, 1, d1);
		fftsg_h.cdft(nfft, 1, 1, d2);
		d2[0] += d1[0];
		xr = d1[1] * d2[1] + d1[2] * d2[2];
		xi = d1[1] * d2[2] + d1[2] * d2[1];
		d2[1] = xr;
		d2[2] = xi;
		if (nfft > 2) {
			mp_mul_rcmul_nt_out(nfft, 1, d1, d2);
		}
		d2[nfft + 1] *= d1[nfft + 1];
	}


	static void mp_mul_cmul_nt_d1_add(int nfft, double d1[], double d2[], double d3[]) {
		double xr, xi;

		fftsg_h.cdft(nfft, 1, 1, d2);
		xr = d1[1] * d2[1] + d1[2] * d2[2];
		xi = d1[1] * d2[2] + d1[2] * d2[1];
		d3[1] += xr;
		d3[2] += xi;
		if (nfft > 2) {
			mp_mul_rcmul_nt_in1_add(nfft, 1, d1, d2, d3);
		}
		d3[nfft + 1] += d1[nfft + 1] * d2[nfft + 1];
		fftsg_h.cdft(nfft, 1, -1, d3);
	}


	static void mp_mul_csqu(int nfft, double d1[]) {
		double xr, xi;

		fftsg_h.cdft(nfft, 1, 1, d1);
		d1[0] *= 2;
		xr = d1[1] * d1[1] + d1[2] * d1[2];
		xi = 2 * d1[1] * d1[2];
		d1[1] = xr;
		d1[2] = xi;
		if (nfft > 2) {
			mp_mul_rcsqu(nfft, 1, d1);
		}
		d1[nfft + 1] *= d1[nfft + 1];
		fftsg_h.cdft(nfft, 1, -1, d1);
	}


	static void mp_mul_csqu_save_d1(int nfft, double d1[], double d2[]) {
		double xr, xi;

		fftsg_h.cdft(nfft, 1, 1, d1);
		d2[0] = 2 * d1[0];
		xr = d1[1] * d1[1] + d1[2] * d1[2];
		xi = 2 * d1[1] * d1[2];
		d2[1] = xr;
		d2[2] = xi;
		if (nfft > 2) {
			mp_mul_rcsqu_save(nfft, 1, d1, d2);
		}
		d2[nfft + 1] = d1[nfft + 1] * d1[nfft + 1];
		fftsg_h.cdft(nfft, 1, -1, d2);
	}


	static void mp_mul_csqu_nt_d1(int nfft, double d1[]) {
		double xr, xi;

		d1[0] *= 2;
		xr = d1[1] * d1[1] + d1[2] * d1[2];
		xi = 2 * d1[1] * d1[2];
		d1[1] = xr;
		d1[2] = xi;
		if (nfft > 2) {
			mp_mul_rcsqu_nt_in(nfft, 1, d1);
		}
		d1[nfft + 1] *= d1[nfft + 1];
		fftsg_h.cdft(nfft, 1, -1, d1);
	}


	static void mp_mul_d2i(int n, int radix, int nfft, double din[], int out[]) {
		int j, carry, carry1, carry2, shift, ndata;
		double x, scale, d1_radix, d1_radix2, pow_radix,
		topdgt;

		scale = 2.0 / nfft;
		d1_radix = 1.0 / radix;
		d1_radix2 = d1_radix * d1_radix;
		topdgt = din[nfft + 1];
		x = topdgt < 0 ? -topdgt : topdgt;
		shift = x + 0.5 >= radix ? 1 : 0;
		/* ---- correction of cyclic convolution of din[1] ---- */
		x *= nfft * 0.5;
		din[nfft + 1] = din[1] - x;
		din[1] = x;
		/* ---- output of digits ---- */
		ndata = n;
		if (n > nfft + 1 + shift) {
			ndata = nfft + 1 + shift;
			for (j = n + 1; j > ndata + 1; j--) {
				out[j] = 0;
			}
		}
		x = 0;
		pow_radix = 1;
		for (j = ndata + 1 - shift; j <= nfft + 1; j++) {
			x += pow_radix * din[j];
			pow_radix *= d1_radix;
			if (pow_radix < DBL_EPSILON) {
				break;
			}
		}
		x = d1_radix2 * (scale * x + 0.5);
		carry2 = ((int) x) - 1;
		carry = (int) (radix * (x - carry2) + 0.5);
		for (j = ndata; j > 1; j--) {
			x = d1_radix2 * (scale * din[j - shift] + carry + 0.5);
			carry = carry2;
			carry2 = ((int) x) - 1;
			x = radix * (x - carry2);
			carry1 = (int) x;
			out[j + 1] = (int) (radix * (x - carry1));
			carry += carry1;
		}
		x = carry + ((double) radix) * carry2 + 0.5;
		if (shift == 0) {
			x += scale * din[1];
		}
		carry = (int) (d1_radix * x);
		out[2] = (int) (x - ((double) radix) * carry);
		if (carry > 0) {
			for (j = n + 1; j > 2; j--) {
				out[j] = out[j - 1];
			}
			out[2] = (int) carry;
			shift++;
		}
		/* ---- output of exp, sgn ---- */
		x = din[0] + shift + 0.5;
		shift = ((int) x) - 1;
		out[1] = shift + ((int) (x - shift));
		out[0] = topdgt > 0.5 ? 1 : -1;
		if (out[2] == 0) {
			out[0] = 0;
			out[1] = 0;
		}
	}


	static double mp_mul_d2i_test(int radix, int nfft, double din[]) {
		int j, carry, carry1, carry2;
		double x, scale, d1_radix, d1_radix2, err;

		scale = 2.0 / nfft;
		d1_radix = 1.0 / radix;
		d1_radix2 = d1_radix * d1_radix;
		/* ---- correction of cyclic convolution of din[1] ---- */
		x = din[nfft + 1] * nfft * 0.5;
		if (x < 0) {
			x = -x;
		}
		din[nfft + 1] = din[1] - x;
		/* ---- check of digits ---- */
		err = 0;
		carry = 0;
		carry2 = 0;
		for (j = nfft + 1; j > 1; j--) {
			x = d1_radix2 * (scale * din[j] + carry + 0.5);
			carry = carry2;
			carry2 = ((int) x) - 1;
			x = radix * (x - carry2);
			carry1 = (int) x;
			x = radix * (x - carry1);
			carry += carry1;
			x = x - 0.5 - ((int) x);
			if (x > err) {
				err = x;
			} else if (-x > err) {
				err = -x;
			}
		}
		return err;
	}


	/* -------- mp_mul child^2 routines (mix RFFT routines) -------- */

	static void mp_mul_rcmul(int n, int base, double a[], double b[]) {
		int i, i0, j, k;
		double ec, w1r, w1i, wkr, wki, wdr, wdi, ss;
		double xr, xi, yr, yi, ajr, aji, akr, aki, bjr,
		bji, bkr, bki;

		ec = 2 * fftsg_h.M_PI_2 / n;
		wkr = 0;
		wki = 0;
		wdi = Math.cos(ec);
		wdr = Math.sin(ec);
		wdi *= wdr;
		wdr *= wdr;
		w1r = 1 - 2 * wdr;
		w1i = 2 * wdi;
		ss = 2 * w1i;
		i = n >> 1;
		xr = a[base + i];
		xi = a[base + i + 1];
		yr = b[base + i];
		yi = b[base + i + 1];
		b[base + i] = xr * yr - xi * yi;
		b[base + i + 1] = xr * yi + xi * yr;
		for (;;) {
			i0 = i - 4 * fftsg_h.RDFT_LOOP_DIV;
			if (i0 < 2) {
				i0 = 2;
			}
			for (j = i - 2; j >= i0; j -= 2) {
				k = n - j;
				xr = wkr + ss * wdi;
				xi = wki + ss * (0.5 - wdr);
				wkr = wdr;
				wki = wdi;
				wdr = xr;
				wdi = xi;
				/*
				 * ---- transform CFFT data a[] into RFFT
				 * data ----
				 */
				xr = a[base + j] - a[base + k];
				xi = a[base + j + 1] + a[base + k + 1];
				yr = wkr * xr - wki * xi;
				yi = wkr * xi + wki * xr;
				ajr = a[base + j] - yr;
				aji = a[base + j + 1] - yi;
				akr = a[base + k] + yr;
				aki = a[base + k + 1] - yi;
				a[base + j] = ajr;
				a[base + j + 1] = aji;
				a[base + k] = akr;
				a[base + k + 1] = aki;
				/*
				 * ---- transform CFFT data b[] into RFFT
				 * data ----
				 */
				xr = b[base + j] - b[base + k];
				xi = b[base + j + 1] + b[base + k + 1];
				yr = wkr * xr - wki * xi;
				yi = wkr * xi + wki * xr;
				xr = b[base + j] - yr;
				xi = b[base + j + 1] - yi;
				yr = b[base + k] + yr;
				yi = b[base + k + 1] - yi;
				/* ---- cmul ---- */
				bjr = ajr * xr - aji * xi;
				bji = ajr * xi + aji * xr;
				bkr = akr * yr - aki * yi;
				bki = akr * yi + aki * yr;
				/*
				 * ---- transform RFFT data bxx into CFFT
				 * data ----
				 */
				xr = bjr - bkr;
				xi = bji + bki;
				yr = wkr * xr + wki * xi;
				yi = wkr * xi - wki * xr;
				b[base + j] = bjr - yr;
				b[base + j + 1] = bji - yi;
				b[base + k] = bkr + yr;
				b[base + k + 1] = bki - yi;
			}
			if (i0 == 2) {
				break;
			}
			wkr = 0.5 * Math.sin(ec * i0);
			wki = 0.5 * Math.cos(ec * i0);
			wdr = 0.5 - (wkr * w1r - wki * w1i);
			wdi = wkr * w1i + wki * w1r;
			wkr = 0.5 - wkr;
			i = i0;
		}
	}


	static void mp_mul_rcmul_nt_in1(int n, int base, double a[], double b[]) {
		int i, i0, j, k;
		double ec, w1r, w1i, wkr, wki, wdr, wdi, ss;
		double xr, xi, yr, yi, bjr, bji, bkr, bki;

		ec = 2 * fftsg_h.M_PI_2 / n;
		wkr = 0;
		wki = 0;
		wdi = Math.cos(ec);
		wdr = Math.sin(ec);
		wdi *= wdr;
		wdr *= wdr;
		w1r = 1 - 2 * wdr;
		w1i = 2 * wdi;
		ss = 2 * w1i;
		i = n >> 1;
		xr = a[base + i];
		xi = a[base + i + 1];
		yr = b[base + i];
		yi = b[base + i + 1];
		b[base + i] = xr * yr - xi * yi;
		b[base + i + 1] = xr * yi + xi * yr;
		for (;;) {
			i0 = i - 4 * fftsg_h.RDFT_LOOP_DIV;
			if (i0 < 2) {
				i0 = 2;
			}
			for (j = i - 2; j >= i0; j -= 2) {
				k = n - j;
				xr = wkr + ss * wdi;
				xi = wki + ss * (0.5 - wdr);
				wkr = wdr;
				wki = wdi;
				wdr = xr;
				wdi = xi;
				/*
				 * ---- transform CFFT data b[base+] into
				 * RFFT data ----
				 */
				xr = b[base + j] - b[base + k];
				xi = b[base + j + 1] + b[base + k + 1];
				yr = wkr * xr - wki * xi;
				yi = wkr * xi + wki * xr;
				xr = b[base + j] - yr;
				xi = b[base + j + 1] - yi;
				yr = b[base + k] + yr;
				yi = b[base + k + 1] - yi;
				/* ---- cmul ---- */
				bjr = a[base + j] * xr - a[base + j + 1] * xi;
				bji = a[base + j] * xi + a[base + j + 1] * xr;
				bkr = a[base + k] * yr - a[base + k + 1] * yi;
				bki = a[base + k] * yi + a[base + k + 1] * yr;
				/*
				 * ---- transform RFFT data bxx into CFFT
				 * data ----
				 */
				xr = bjr - bkr;
				xi = bji + bki;
				yr = wkr * xr + wki * xi;
				yi = wkr * xi - wki * xr;
				b[base + j] = bjr - yr;
				b[base + j + 1] = bji - yi;
				b[base + k] = bkr + yr;
				b[base + k + 1] = bki - yi;
			}
			if (i0 == 2) {
				break;
			}
			wkr = 0.5 * Math.sin(ec * i0);
			wki = 0.5 * Math.cos(ec * i0);
			wdr = 0.5 - (wkr * w1r - wki * w1i);
			wdi = wkr * w1i + wki * w1r;
			wkr = 0.5 - wkr;
			i = i0;
		}
	}


	static void mp_mul_rcmul_nt_in2(int n, int base, double a[], double b[]) {
		int i, i0, j, k;
		double ec, w1r, w1i, wkr, wki, wdr, wdi, ss;
		double xr, xi, yr, yi, bjr, bji, bkr, bki;

		ec = 2 * fftsg_h.M_PI_2 / n;
		wkr = 0;
		wki = 0;
		wdi = Math.cos(ec);
		wdr = Math.sin(ec);
		wdi *= wdr;
		wdr *= wdr;
		w1r = 1 - 2 * wdr;
		w1i = 2 * wdi;
		ss = 2 * w1i;
		i = n >> 1;
		xr = a[base + i];
		xi = a[base + i + 1];
		yr = b[base + i];
		yi = b[base + i + 1];
		b[base + i] = xr * yr - xi * yi;
		b[base + i + 1] = xr * yi + xi * yr;
		for (;;) {
			i0 = i - 4 * fftsg_h.RDFT_LOOP_DIV;
			if (i0 < 2) {
				i0 = 2;
			}
			for (j = i - 2; j >= i0; j -= 2) {
				k = n - j;
				xr = wkr + ss * wdi;
				xi = wki + ss * (0.5 - wdr);
				wkr = wdr;
				wki = wdi;
				wdr = xr;
				wdi = xi;
				/*
				 * ---- transform CFFT data a[base+] into
				 * RFFT data ----
				 */
				xr = a[base + j] - a[base + k];
				xi = a[base + j + 1] + a[base + k + 1];
				yr = wkr * xr - wki * xi;
				yi = wkr * xi + wki * xr;
				xr = a[base + j] - yr;
				xi = a[base + j + 1] - yi;
				yr = a[base + k] + yr;
				yi = a[base + k + 1] - yi;
				a[base + j] = xr;
				a[base + j + 1] = xi;
				a[base + k] = yr;
				a[base + k + 1] = yi;
				/* ---- cmul ---- */
				bjr = b[base + j] * xr - b[base + j + 1] * xi;
				bji = b[base + j] * xi + b[base + j + 1] * xr;
				bkr = b[base + k] * yr - b[base + k + 1] * yi;
				bki = b[base + k] * yi + b[base + k + 1] * yr;
				/*
				 * ---- transform RFFT data bxx into CFFT
				 * data ----
				 */
				xr = bjr - bkr;
				xi = bji + bki;
				yr = wkr * xr + wki * xi;
				yi = wkr * xi - wki * xr;
				b[base + j] = bjr - yr;
				b[base + j + 1] = bji - yi;
				b[base + k] = bkr + yr;
				b[base + k + 1] = bki - yi;
			}
			if (i0 == 2) {
				break;
			}
			wkr = 0.5 * Math.sin(ec * i0);
			wki = 0.5 * Math.cos(ec * i0);
			wdr = 0.5 - (wkr * w1r - wki * w1i);
			wdi = wkr * w1i + wki * w1r;
			wkr = 0.5 - wkr;
			i = i0;
		}
	}


	static void mp_mul_rcmul_nt_out(int n, int base, double a[], double b[]) {
		int i, i0, j, k;
		double ec, w1r, w1i, wkr, wki, wdr, wdi, ss;
		double xr, xi, yr, yi, ajr, aji, akr, aki;

		ec = 2 * fftsg_h.M_PI_2 / n;
		wkr = 0;
		wki = 0;
		wdi = Math.cos(ec);
		wdr = Math.sin(ec);
		wdi *= wdr;
		wdr *= wdr;
		w1r = 1 - 2 * wdr;
		w1i = 2 * wdi;
		ss = 2 * w1i;
		i = n >> 1;
		xr = a[base + i];
		xi = a[base + i + 1];
		yr = b[base + i];
		yi = b[base + i + 1];
		b[base + i] = xr * yr - xi * yi;
		b[base + i + 1] = xr * yi + xi * yr;
		for (;;) {
			i0 = i - 4 * fftsg_h.RDFT_LOOP_DIV;
			if (i0 < 2) {
				i0 = 2;
			}
			for (j = i - 2; j >= i0; j -= 2) {
				k = n - j;
				xr = wkr + ss * wdi;
				xi = wki + ss * (0.5 - wdr);
				wkr = wdr;
				wki = wdi;
				wdr = xr;
				wdi = xi;
				/*
				 * ---- transform CFFT data a[base+] into
				 * RFFT data ----
				 */
				xr = a[base + j] - a[base + k];
				xi = a[base + j + 1] + a[base + k + 1];
				yr = wkr * xr - wki * xi;
				yi = wkr * xi + wki * xr;
				ajr = a[base + j] - yr;
				aji = a[base + j + 1] - yi;
				akr = a[base + k] + yr;
				aki = a[base + k + 1] - yi;
				a[base + j] = ajr;
				a[base + j + 1] = aji;
				a[base + k] = akr;
				a[base + k + 1] = aki;
				/*
				 * ---- transform CFFT data b[base+] into
				 * RFFT data ----
				 */
				xr = b[base + j] - b[base + k];
				xi = b[base + j + 1] + b[base + k + 1];
				yr = wkr * xr - wki * xi;
				yi = wkr * xi + wki * xr;
				xr = b[base + j] - yr;
				xi = b[base + j + 1] - yi;
				yr = b[base + k] + yr;
				yi = b[base + k + 1] - yi;
				/* ---- cmul ---- */
				b[base + j] = ajr * xr - aji * xi;
				b[base + j + 1] = ajr * xi + aji * xr;
				b[base + k] = akr * yr - aki * yi;
				b[base + k + 1] = akr * yi + aki * yr;
			}
			if (i0 == 2) {
				break;
			}
			wkr = 0.5 * Math.sin(ec * i0);
			wki = 0.5 * Math.cos(ec * i0);
			wdr = 0.5 - (wkr * w1r - wki * w1i);
			wdi = wkr * w1i + wki * w1r;
			wkr = 0.5 - wkr;
			i = i0;
		}
	}


	static void mp_mul_rcmul_nt_in1_add(int n, int base, double a[], double b[], double badd[]) {
		int i, i0, j, k;
		double ec, w1r, w1i, wkr, wki, wdr, wdi, ss;
		double xr, xi, yr, yi, bjr, bji, bkr, bki;

		ec = 2 * fftsg_h.M_PI_2 / n;
		wkr = 0;
		wki = 0;
		wdi = Math.cos(ec);
		wdr = Math.sin(ec);
		wdi *= wdr;
		wdr *= wdr;
		w1r = 1 - 2 * wdr;
		w1i = 2 * wdi;
		ss = 2 * w1i;
		i = n >> 1;
		xr = a[base + i];
		xi = a[base + i + 1];
		yr = b[base + i];
		yi = b[base + i + 1];
		badd[base + i] += xr * yr - xi * yi;
		badd[base + i + 1] += xr * yi + xi * yr;
		for (;;) {
			i0 = i - 4 * fftsg_h.RDFT_LOOP_DIV;
			if (i0 < 2) {
				i0 = 2;
			}
			for (j = i - 2; j >= i0; j -= 2) {
				k = n - j;
				xr = wkr + ss * wdi;
				xi = wki + ss * (0.5 - wdr);
				wkr = wdr;
				wki = wdi;
				wdr = xr;
				wdi = xi;
				/*
				 * ---- transform CFFT data b[base+] into
				 * RFFT data ----
				 */
				xr = b[base + j] - b[base + k];
				xi = b[base + j + 1] + b[base + k + 1];
				yr = wkr * xr - wki * xi;
				yi = wkr * xi + wki * xr;
				xr = b[base + j] - yr;
				xi = b[base + j + 1] - yi;
				yr = b[base + k] + yr;
				yi = b[base + k + 1] - yi;
				/* ---- cmul + add ---- */
				bjr = badd[base + j] + (a[base + j] * xr - a[base + j + 1] * xi);
				bji = badd[base + j + 1] + (a[base + j] * xi + a[base + j + 1] * xr);
				bkr = badd[base + k] + (a[base + k] * yr - a[base + k + 1] * yi);
				bki = badd[base + k + 1] + (a[base + k] * yi + a[base + k + 1] * yr);
				/*
				 * ---- transform RFFT data bxx into CFFT
				 * data ----
				 */
				xr = bjr - bkr;
				xi = bji + bki;
				yr = wkr * xr + wki * xi;
				yi = wkr * xi - wki * xr;
				badd[base + j] = bjr - yr;
				badd[base + j + 1] = bji - yi;
				badd[base + k] = bkr + yr;
				badd[base + k + 1] = bki - yi;
			}
			if (i0 == 2) {
				break;
			}
			wkr = 0.5 * Math.sin(ec * i0);
			wki = 0.5 * Math.cos(ec * i0);
			wdr = 0.5 - (wkr * w1r - wki * w1i);
			wdi = wkr * w1i + wki * w1r;
			wkr = 0.5 - wkr;
			i = i0;
		}
	}


	static void mp_mul_rcsqu(int n, int base, double a[]) {
		int i, i0, j, k;
		double ec, w1r, w1i, wkr, wki, wdr, wdi, ss;
		double xr, xi, yr, yi, ajr, aji, akr, aki;

		ec = 2 * fftsg_h.M_PI_2 / n;
		wkr = 0;
		wki = 0;
		wdi = Math.cos(ec);
		wdr = Math.sin(ec);
		wdi *= wdr;
		wdr *= wdr;
		w1r = 1 - 2 * wdr;
		w1i = 2 * wdi;
		ss = 2 * w1i;
		i = n >> 1;
		xr = a[base + i];
		xi = a[base + i + 1];
		a[base + i] = xr * xr - xi * xi;
		a[base + i + 1] = 2 * xr * xi;
		for (;;) {
			i0 = i - 4 * fftsg_h.RDFT_LOOP_DIV;
			if (i0 < 2) {
				i0 = 2;
			}
			for (j = i - 2; j >= i0; j -= 2) {
				k = n - j;
				xr = wkr + ss * wdi;
				xi = wki + ss * (0.5 - wdr);
				wkr = wdr;
				wki = wdi;
				wdr = xr;
				wdi = xi;
				/*
				 * ---- transform CFFT data a[base+] into
				 * RFFT data ----
				 */
				xr = a[base + j] - a[base + k];
				xi = a[base + j + 1] + a[base + k + 1];
				yr = wkr * xr - wki * xi;
				yi = wkr * xi + wki * xr;
				xr = a[base + j] - yr;
				xi = a[base + j + 1] - yi;
				yr = a[base + k] + yr;
				yi = a[base + k + 1] - yi;
				/* ---- csqu ---- */
				ajr = xr * xr - xi * xi;
				aji = 2 * xr * xi;
				akr = yr * yr - yi * yi;
				aki = 2 * yr * yi;
				/*
				 * ---- transform RFFT data axx into CFFT
				 * data ----
				 */
				xr = ajr - akr;
				xi = aji + aki;
				yr = wkr * xr + wki * xi;
				yi = wkr * xi - wki * xr;
				a[base + j] = ajr - yr;
				a[base + j + 1] = aji - yi;
				a[base + k] = akr + yr;
				a[base + k + 1] = aki - yi;
			}
			if (i0 == 2) {
				break;
			}
			wkr = 0.5 * Math.sin(ec * i0);
			wki = 0.5 * Math.cos(ec * i0);
			wdr = 0.5 - (wkr * w1r - wki * w1i);
			wdi = wkr * w1i + wki * w1r;
			wkr = 0.5 - wkr;
			i = i0;
		}
	}


	static void mp_mul_rcsqu_save(int n, int base, double a[], double b[]) {
		int i, i0, j, k;
		double ec, w1r, w1i, wkr, wki, wdr, wdi, ss;
		double xr, xi, yr, yi, ajr, aji, akr, aki;

		ec = 2 * fftsg_h.M_PI_2 / n;
		wkr = 0;
		wki = 0;
		wdi = Math.cos(ec);
		wdr = Math.sin(ec);
		wdi *= wdr;
		wdr *= wdr;
		w1r = 1 - 2 * wdr;
		w1i = 2 * wdi;
		ss = 2 * w1i;
		i = n >> 1;
		xr = a[base + i];
		xi = a[base + i + 1];
		b[base + i] = xr * xr - xi * xi;
		b[base + i + 1] = 2 * xr * xi;
		for (;;) {
			i0 = i - 4 * fftsg_h.RDFT_LOOP_DIV;
			if (i0 < 2) {
				i0 = 2;
			}
			for (j = i - 2; j >= i0; j -= 2) {
				k = n - j;
				xr = wkr + ss * wdi;
				xi = wki + ss * (0.5 - wdr);
				wkr = wdr;
				wki = wdi;
				wdr = xr;
				wdi = xi;
				/*
				 * ---- transform CFFT data a[base+] into
				 * RFFT data ----
				 */
				xr = a[base + j] - a[base + k];
				xi = a[base + j + 1] + a[base + k + 1];
				yr = wkr * xr - wki * xi;
				yi = wkr * xi + wki * xr;
				xr = a[base + j] - yr;
				xi = a[base + j + 1] - yi;
				yr = a[base + k] + yr;
				yi = a[base + k + 1] - yi;
				a[base + j] = xr;
				a[base + j + 1] = xi;
				a[base + k] = yr;
				a[base + k + 1] = yi;
				/* ---- csqu ---- */
				ajr = xr * xr - xi * xi;
				aji = 2 * xr * xi;
				akr = yr * yr - yi * yi;
				aki = 2 * yr * yi;
				/*
				 * ---- transform RFFT data axx into CFFT
				 * data ----
				 */
				xr = ajr - akr;
				xi = aji + aki;
				yr = wkr * xr + wki * xi;
				yi = wkr * xi - wki * xr;
				b[base + j] = ajr - yr;
				b[base + j + 1] = aji - yi;
				b[base + k] = akr + yr;
				b[base + k + 1] = aki - yi;
			}
			if (i0 == 2) {
				break;
			}
			wkr = 0.5 * Math.sin(ec * i0);
			wki = 0.5 * Math.cos(ec * i0);
			wdr = 0.5 - (wkr * w1r - wki * w1i);
			wdi = wkr * w1i + wki * w1r;
			wkr = 0.5 - wkr;
			i = i0;
		}
	}


	static void mp_mul_rcsqu_nt_in(int n, int base, double a[]) {
		int i, i0, j, k;
		double ec, w1r, w1i, wkr, wki, wdr, wdi, ss;
		double xr, xi, yr, yi, ajr, aji, akr, aki;

		ec = 2 * fftsg_h.M_PI_2 / n;
		wkr = 0;
		wki = 0;
		wdi = Math.cos(ec);
		wdr = Math.sin(ec);
		wdi *= wdr;
		wdr *= wdr;
		w1r = 1 - 2 * wdr;
		w1i = 2 * wdi;
		ss = 2 * w1i;
		i = n >> 1;
		xr = a[base + i];
		xi = a[base + i + 1];
		a[base + i] = xr * xr - xi * xi;
		a[base + i + 1] = 2 * xr * xi;
		for (;;) {
			i0 = i - 4 * fftsg_h.RDFT_LOOP_DIV;
			if (i0 < 2) {
				i0 = 2;
			}
			for (j = i - 2; j >= i0; j -= 2) {
				k = n - j;
				xr = wkr + ss * wdi;
				xi = wki + ss * (0.5 - wdr);
				wkr = wdr;
				wki = wdi;
				wdr = xr;
				wdi = xi;
				/* ---- csqu ---- */
				xr = a[base + j];
				xi = a[base + j + 1];
				yr = a[base + k];
				yi = a[base + k + 1];
				ajr = xr * xr - xi * xi;
				aji = 2 * xr * xi;
				akr = yr * yr - yi * yi;
				aki = 2 * yr * yi;
				/*
				 * ---- transform RFFT data axx into CFFT
				 * data ----
				 */
				xr = ajr - akr;
				xi = aji + aki;
				yr = wkr * xr + wki * xi;
				yi = wkr * xi - wki * xr;
				a[base + j] = ajr - yr;
				a[base + j + 1] = aji - yi;
				a[base + k] = akr + yr;
				a[base + k + 1] = aki - yi;
			}
			if (i0 == 2) {
				break;
			}
			wkr = 0.5 * Math.sin(ec * i0);
			wki = 0.5 * Math.cos(ec * i0);
			wdr = 0.5 - (wkr * w1r - wki * w1i);
			wdi = wkr * w1i + wki * w1r;
			wkr = 0.5 - wkr;
			i = i0;
		}
	}


	/* -------- mp_inv routines -------- */


	static int mp_inv(int n, int radix, int in[], int out[],
		int tmp1[], int tmp2[], int nfft, double tmp1fft[], double tmp2fft[]) {
		int n_nwt, nfft_nwt, thr, prc;

		if (in[0] == 0) {
			return -1;
		}
		nfft_nwt = mp_get_nfft_init(radix, nfft);
		n_nwt = nfft_nwt + 2;
		if (n_nwt > n) {
			n_nwt = n;
		}
		mp_inv_init(n_nwt, radix, in, out);
		thr = 8;
		do {
			n_nwt = nfft_nwt + 2;
			if (n_nwt > n) {
				n_nwt = n;
			}
			prc = mp_inv_newton(n_nwt, radix, in, out,
				tmp1, tmp2, nfft_nwt, tmp1fft, tmp2fft);
			if (thr * nfft_nwt >= nfft) {
				thr = 0;
				if (2 * prc <= n_nwt - 2) {
					nfft_nwt >>= 1;
				}
			} else {
				if (3 * prc < n_nwt - 2) {
					nfft_nwt >>= 1;
				}
			}
			nfft_nwt <<= 1;
		}
		while (nfft_nwt <= nfft);
		return 0;
	}


	static int mp_sqrt(int n, int radix, int in[], int out[],
				int tmp1[], int tmp2[], int nfft, double tmp1fft[], double tmp2fft[]) {
		int n_nwt, nfft_nwt, thr, prc;
		int n_tmp1fft[] = new int[1];

		if (in[0] < 0) {
			return -1;
		} else if (in[0] == 0) {
			mp_load_0(n, radix, out);
			return 0;
		}
		nfft_nwt = mp_get_nfft_init(radix, nfft);
		n_nwt = nfft_nwt + 2;
		if (n_nwt > n) {
			n_nwt = n;
		}
		mp_sqrt_init(n_nwt, radix, in, out, tmp1);
		n_tmp1fft[0] = 0;
		thr = 8;
		do {
			n_nwt = nfft_nwt + 2;
			if (n_nwt > n) {
				n_nwt = n;
			}
			prc = mp_sqrt_newton(n_nwt, radix, in, out,
				tmp1, tmp2, nfft_nwt, tmp1fft, tmp2fft,
				n_tmp1fft);
			if (thr * nfft_nwt >= nfft) {
				thr = 0;
				if (2 * prc <= n_nwt - 2) {
					nfft_nwt >>= 1;
				}
			} else {
				if (3 * prc < n_nwt - 2) {
					nfft_nwt >>= 1;
				}
			}
			nfft_nwt <<= 1;
		}
		while (nfft_nwt <= nfft);
		return 0;
	}


	static int mp_invisqrt(int n, int radix, int in, int out[],
		int tmp1[], int tmp2[], int nfft,
		double tmp1fft[], double tmp2fft[]) {
		int n_nwt, nfft_nwt, thr, prc;

		if (in <= 0) {
			return -1;
		}
		nfft_nwt = mp_get_nfft_init(radix, nfft);
		n_nwt = nfft_nwt + 2;
		if (n_nwt > n) {
			n_nwt = n;
		}
		mp_invisqrt_init(n_nwt, radix, in, out);
		thr = 8;
		do {
			n_nwt = nfft_nwt + 2;
			if (n_nwt > n) {
				n_nwt = n;
			}
			prc = mp_invisqrt_newton(n_nwt, radix, in, out,
				tmp1, tmp2, nfft_nwt, tmp1fft, tmp2fft);
			if (thr * nfft_nwt >= nfft) {
				thr = 0;
				if (2 * prc <= n_nwt - 2) {
					nfft_nwt >>= 1;
				}
			} else {
				if (3 * prc < n_nwt - 2) {
					nfft_nwt >>= 1;
				}
			}
			nfft_nwt <<= 1;
		}
		while (nfft_nwt <= nfft);
		return 0;
	}


	/* -------- mp_inv child routines -------- */


	static int mp_get_nfft_init(int radix, int nfft_max) {
		int nfft_init;
		double r;

		r = radix;
		nfft_init = 1;
		do {
			r *= r;
			nfft_init <<= 1;
		}
		while (DBL_EPSILON * r < 1 && nfft_init < nfft_max);
		return nfft_init;
	}


	static void mp_inv_init(int n, int radix, int in[], int out[]) {
		int outexp;
		double din;

		out[0] = in[0];
		outexp = -in[1];
		din = 1.0 / mp_unexp_mp2d(n, 2, radix, in);
		while (din < 1) {
			din *= radix;
			outexp--;
		}
		out[1] = outexp;
		mp_unexp_d2mp(n, 2, radix, din, out);
	}


	static void mp_sqrt_init(int n, int radix, int in[], int out[], int out_rev[]) {
		int outexp;
		double din;

		out[0] = 1;
		out_rev[0] = 1;
		outexp = in[1];
		din = mp_unexp_mp2d(n, 2, radix, in);
		if (outexp % 2 != 0) {
			din *= radix;
			outexp--;
		}
		outexp /= 2;
		din = Math.sqrt(din);
		if (din < 1) {
			din *= radix;
			outexp--;
		}
		out[1] = outexp;
		mp_unexp_d2mp(n, 2, radix, din, out);
		outexp = -outexp;
		din = 1.0 / din;
		while (din < 1) {
			din *= radix;
			outexp--;
		}
		out_rev[1] = outexp;
		mp_unexp_d2mp(n, 2, radix, din, out_rev);
	}


	static void mp_invisqrt_init(int n, int radix, int in, int out[]) {
		int outexp;
		double dout;

		out[0] = 1;
		outexp = 0;
		dout = Math.sqrt(1.0 / in);
		while (dout < 1) {
			dout *= radix;
			outexp--;
		}
		out[1] = outexp;
		mp_unexp_d2mp(n, 2, radix, dout, out);
	}


	static void mp_unexp_d2mp(int n, int base, int radix, double din, int out[]) {
		int j, x;

		for (j = 0; j < n; j++) {
			x = (int) din;
			if (x >= radix) {
				x = radix - 1;
				din = radix;
			}
			din = radix * (din - x);
			out[base + j] = (int) x;
		}
	}


	static double mp_unexp_mp2d(int n, int base, int radix, int in[]) {
		int j;
		double d1_radix, dout;

		d1_radix = 1.0 / radix;
		dout = 0;
		for (j = n - 1; j >= 0; j--) {
			dout = d1_radix * dout + in[base + j];
		}
		return dout;
	}


	static int mp_inv_newton(int n, int radix, int in[], int inout[],
		int tmp1[], int tmp2[], int nfft, double tmp1fft[],
		double tmp2fft[]) {
		int n_h, shift, prc;

		shift = (nfft >> 1) + 1;
		n_h = n / 2 + 1;
		if (n_h < n - shift) {
			n_h = n - shift;
		}
		/*
		 * ---- tmp1 = inout * (upper) in (half to normal precision)
		 * ----
		 */
		mp_round(n, radix, shift, inout);
		mp_mulh(n, radix, inout, in, tmp1, nfft, tmp1fft, tmp2fft);
		/* ---- tmp2 = 1 - tmp1 ---- */
		mp_load_1(n, radix, tmp2);
		mp_sub(n, radix, tmp2, tmp1, tmp2);
		/* ---- tmp2 -= inout * (lower) in (half precision) ---- */
		mp_mulh_use_in1fft(n, radix, tmp1fft, shift, in, tmp1, nfft, tmp2fft);
		mp_sub(n_h, radix, tmp2, tmp1, tmp2);
		/* ---- get precision ---- */
		prc = -tmp2[1];
		if (tmp2[0] == 0) {
			prc = nfft + 1;
		}
		/* ---- tmp2 *= inout (half precision) ---- */
		mp_mulh_use_in1fft(n_h, radix, tmp1fft, 0, tmp2, tmp2, nfft, tmp2fft);
		/* ---- inout += tmp2 ---- */
		mp_add(n, radix, inout, tmp2, inout);
		return prc;
	}


	static int mp_sqrt_newton(int n, int radix, int in[], int inout[],
		int inout_rev[], int tmp[], int nfft, double tmp1fft[],
		double tmp2fft[], int n_tmp1fft[]) {
		int n_h, nfft_h, shift, prc;

		nfft_h = nfft >> 1;
		shift = nfft_h + 1;
		if (nfft_h < 2) {
			nfft_h = 2;
		}
		n_h = n / 2 + 1;
		if (n_h < n - shift) {
			n_h = n - shift;
		}
		/* ---- tmp = inout_rev^2 (1/4 to half precision) ---- */
		mp_round(n_h, radix, (nfft_h >> 1) + 1, inout_rev);
		if (n_tmp1fft[0] != nfft_h) {
			mp_squh(n_h, radix, inout_rev, tmp, nfft_h, tmp1fft);
		} else {
			mp_squh_use_in1fft(n_h, radix, tmp1fft, tmp, nfft_h);
		}
		/* ---- tmp = inout_rev - inout * tmp (half precision) ---- */
		mp_round(n, radix, shift, inout);
		mp_mulh(n_h, radix, inout, tmp, tmp, nfft, tmp1fft, tmp2fft);
		mp_sub(n_h, radix, inout_rev, tmp, tmp);
		/* ---- inout_rev += tmp ---- */
		mp_add(n_h, radix, inout_rev, tmp, inout_rev);
		/* ---- tmp = in - inout^2 (half to normal precision) ---- */
		mp_squh_use_in1fft(n, radix, tmp1fft, tmp, nfft);
		mp_sub(n, radix, in, tmp, tmp);
		/* ---- get precision ---- */
		prc = in[1] - tmp[1];
		if (in[2] > tmp[2]) {
			prc++;
		}
		if (tmp[0] == 0) {
			prc = nfft + 1;
		}
		/* ---- tmp = tmp * inout_rev / 2 (half precision) ---- */
		mp_round(n_h, radix, shift, inout_rev);
		mp_mulh(n_h, radix, inout_rev, tmp, tmp, nfft, tmp1fft, tmp2fft);
		n_tmp1fft[0] = nfft;
		mp_idiv_2(n_h, radix, tmp, tmp);
		/* ---- inout += tmp ---- */
		mp_add(n, radix, inout, tmp, inout);
		return prc;
	}


	static int mp_invisqrt_newton(int n, int radix, int in, int inout[],
		int tmp1[], int tmp2[], int nfft, double tmp1fft[],
		double tmp2fft[]) {
		int n_h, shift, prc;

		shift = (nfft >> 1) + 1;
		n_h = n / 2 + 1;
		if (n_h < n - shift) {
			n_h = n - shift;
		}
		/* ---- tmp1 = in * inout^2 (half to normal precision) ---- */
		mp_round(n, radix, shift, inout);
		mp_squh_save_infft(n, radix, inout, tmp1, nfft, tmp1fft, tmp2fft);
		mp_imul(n, radix, tmp1, in, tmp1);
		/* ---- tmp2 = 1 - tmp1 ---- */
		mp_load_1(n, radix, tmp2);
		mp_sub(n, radix, tmp2, tmp1, tmp2);
		/* ---- get precision ---- */
		prc = -tmp2[1];
		if (tmp2[0] == 0) {
			prc = nfft + 1;
		}
		/* ---- tmp2 *= inout / 2 (half precision) ---- */
		mp_mulh_use_in1fft(n_h, radix, tmp1fft, 0, tmp2, tmp2, nfft, tmp2fft);
		mp_idiv_2(n_h, radix, tmp2, tmp2);
		/* ---- inout += tmp2 ---- */
		mp_add(n, radix, inout, tmp2, inout);
		return prc;
	}


	/* -------- mp_io routines -------- */


	static void mp_sprintf(int n, int log10_radix, int in[], char out[]) {
		String s;
		int j, k, x, y, outexp, shift, count = 0;

		if (in[0] < 0) {
			out[0] = '-';
			count++;
		}
		x = in[2];
		shift = log10_radix;
		for (k = log10_radix; k > 0; k--) {
			y = x % 10;
			x /= 10;
			s = "" + y;
			out[count + k] = s.charAt(0);
			if (y != 0) {
				shift = k;
			}
		}
		out[count] = out[count + shift];
		out[count + 1] = '.';
		for (k = 1; k <= log10_radix - shift; k++) {
			out[count + k + 1] = out[count + k + shift];
		}
		outexp = log10_radix - shift;
		count += outexp + 2;
		for (j = 3; j <= n + 1; j++) {
			x = in[j];
			for (k = log10_radix - 1; k >= 0; k--) {
				y = x % 10;
				x /= 10;
				s = "" + y;
				out[count + k] = s.charAt(0);
			}
			count += log10_radix;
		}
		out[count] = 'e';
		out[count + 1] = '0';
		out[count + 2] = '\0';
	}


	static void mp_sscanf(int n, int log10_radix, String in, int out[]) {
		int i = 0, j, k, x, outexp, outexp_mod, end = in.length();

		out[0] = 1;

		while (in.charAt(i) == '0')
			i++;
		outexp = 0;

		if (in.charAt(i) == '.') {
			do {
				outexp--;
				i++;
			} while (i != end && in.charAt(i) == '0');
		}

			else if (i != end) {
			for (k = i + 1; k != end && in.charAt(k) >= '0' && in.charAt(k) <= '9'; k++)
				outexp++;
		}
		x = outexp / log10_radix;
		outexp_mod = outexp - log10_radix * x;
		if (outexp_mod < 0) {
			x--;
			outexp_mod += log10_radix;
		}
		out[1] = x;
		x = 0;
		j = 2;

		for (k = i; k != end; k++) {
			if (in.charAt(k) == '.') {
				continue;
			}
			x = 10 * x + (in.charAt(k) - '0');
			if (--outexp_mod < 0) {
				if (j > n + 1) {
					break;
				}
				out[j++] = x;
				x = 0;
				outexp_mod = log10_radix - 1;
			}
		}


		while (outexp_mod-- >= 0)
			x *= 10;

			while (j <= n + 1) {
			out[j++] = x;
			x = 0;
		}
	}

}
